Introduction
------------

Crawler Rate Limit allows you to limit requests performed by web crawlers, bots and spiders.

There are number of crawlers, bots and spiders that are known for excessive crawling of websites. Such requests increase the server load and often affect performance of the website for regular visitors. In extreme cases they can even bring the site down.

This module detects if the request is made by the crawler/bot/spider by inspecting the UserAgent HTTP header and then limits number of requests crawler is allowed to perform in the given time interval. After the limit is exceeded server will respond with HTTP code 429 (Too many requests). Crawler is unblocked at the end of that time interval and new cycle beings. If crawler exceeds the rate limit again, it will again be blocked for the duration of that time interval.

**Features**
- targets only web crawlers, bots and spiders; regular browser traffic is not affected
- number of allowed requests in a given time interval is configurable
- limits requests based on the UserAgent string - if the same crawler uses multiple IP addresses all those requests will count towards the single limit
- minimal effect on performance
  - rate limiting is performed in a very early phase of request handling
  - uses Redis as a rate limiter backend

Requirements
------------

- Redis Drupal module - https://www.drupal.org/project/redis
- jaybizzle/crawler-detect - https://github.com/jaybizzle/crawler-detect
- nikolaposa/rate-limit - https://github.com/nikolaposa/rate-limit


Installation
------------

Crawler Rate Limit manages its dependencies via composer. Just copying the module into the modules folder won't work. Use composer to install the module along with all the dependencies.

```
cd /root/of/your/drupal/project
composer require drupal/crawler_rate_limit
```

Configuration
-------------

1. If not already enabled, enable and configure Redis Drupal module.

2. Enable the `crawler_rate_limit` module and add the following snippet to your `settings.php` file.

```php
// Below configuration will allow crawlers to perform 100 requests
// within 600 seconds.

// Time interval in seconds.
$settings['crawler_rate_limit.settings']['interval'] = 600;
// Number of requests allowed in the given time interval.
$settings['crawler_rate_limit.settings']['operations'] = 100;
// Set to FALSE to disable the limiter.
$settings['crawler_rate_limit.settings']['enabled'] = TRUE;
```

Make sure to adjust the values for `interval` and `operations` according to the capabilities of your server and level of the traffic your website receives. You may need to review your server logs in order to determine optimal values for your server.

Maintainers
-----------
- Vojislav Jovanović (vaish) - https://www.drupal.org/u/vaish


Supporting organizations:
- CED Greentech - https://www.drupal.org/ced-greentech
